from django.db import models
from django.utils.translation import gettext_lazy as _


class Property(models.Model):
    class Type(models.TextChoices):
        QUANTITY = "quantity", _("Quantity")
        TEXT = "text", _("Text")

    name = models.CharField(max_length=50)
    filterable = models.BooleanField(
        help_text=_("Whether this property can be used to filter components")
    )
    type = models.CharField(choices=Type, max_length=8)
    searchable = models.BooleanField(
        help_text=_("Whether this property should be used in search")
    )
    unit = models.CharField(
        max_length=3, help_text=_("Symbol of this quantity's unit of measurement")
    )
    scale = models.SmallIntegerField(
        help_text=_("Exponent of the smallest possible value")
    )

    def __str__(self):
        return self.name + (
            f" [{self.unit}]" if self.type == Property.Type.QUANTITY else ""
        )


class Component(models.Model):
    part_number = models.CharField(max_length=30)
    properties = models.ManyToManyField(Property, through="Value")

    def __str__(self):
        return self.part_number


class Value(models.Model):
    class Meta:
        unique_together = ("component", "property")

    property = models.ForeignKey(Property, on_delete=models.CASCADE)
    component = models.ForeignKey(Component, on_delete=models.CASCADE)
    text = models.TextField()
    integer = models.BigIntegerField()

    def __str__(self):
        match self.property.type:
            case Property.Type.QUANTITY:
                s = str(self.integer)

                scale = self.property.scale
                if scale > 0:
                    s += "0" * scale
                elif scale < 0:
                    s = str(self.integer).rjust(abs(scale) + 1, "0")
                    s = s[:scale] + "." + s[scale:]

                return f"{s} {self.property.unit}"
            case Property.Type.TEXT:
                return self.text
