from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.db import transaction
from django.forms.models import model_to_dict
from django.http import QueryDict
from django.shortcuts import redirect, render
from django.urls import reverse
from django.utils.translation import ngettext

from .forms import AddStockFormSet
from .models import Component, Property, Stock, User, Value


@login_required
def index(request):
    return render(request, "core/index.html", {})


def unauthorized(request):
    return render(request, "core/unauthorized.html", {})


@login_required
def add_stock(request):
    property_models = {p.name: p for p in Property.objects.all()}

    if request.method == "POST":
        formset = AddStockFormSet(request.POST, initial=[{"owner": request.user}])
        if not formset.is_valid():
            return render(request, "core/add_stock.html", {"formset": formset})

        try:
            with transaction.atomic():
                for form in formset:
                    part_number = form.cleaned_data["part_number"]
                    component, created = Component.objects.get_or_create(
                        part_number__iexact=part_number,
                        defaults={"part_number": part_number},
                    )

                    if created:
                        properties = form.cleaned_data["properties"]
                        for name in properties:
                            if name not in property_models:
                                property_kwargs = {
                                    k: v
                                    for k, v in properties[name].items()
                                    if k
                                    in (
                                        "filterable",
                                        "type",
                                        "searchable",
                                        "unit",
                                        "scale",
                                    )
                                }
                                property_models[name] = Property.objects.create(
                                    name=name, **property_kwargs
                                )
                            property = property_models[name]

                            value_kwargs = {
                                k: v
                                for k, v in properties[name].items()
                                if k in ("integer", "text")
                            }
                            value_kwargs.setdefault("integer", 0)
                            value_kwargs.setdefault("text", "")
                            Value.objects.create(
                                property=property, component=component, **value_kwargs
                            )

                    owner_name = form.cleaned_data["owner"]
                    owner = User.objects.filter(username=owner_name).first()

                    Stock.objects.create(
                        component=component,
                        quantity=form.cleaned_data["quantity"],
                        original_quantity=form.cleaned_data["original_quantity"],
                        total_value=form.cleaned_data["total_value"],
                        location=form.cleaned_data["location"],
                        owner=owner,
                        owner_name=owner_name,
                    )
        except Exception as e:
            messages.add_message(request, messages.ERROR, str(e))

            return render(
                request,
                "core/add_stock.html",
                {
                    "formset": formset,
                    "properties": list(map(model_to_dict, property_models.values())),
                },
            )

        messages.add_message(
            request,
            messages.SUCCESS,
            ngettext(
                formset.total_form_count,
                "Added %(count)d stock entry",
                "Added %(count)d stock entries",
            )
            % {"count": formset.total_form_count},
        )

    return render(
        request,
        "core/add_stock.html",
        {
            "formset": AddStockFormSet(initial=[{"owner": request.user}]),
            "properties": list(map(model_to_dict, property_models.values())),
        },
    )


def admin_login(request):
    if request.user.is_authenticated:
        return redirect("unauthorized")

    query = ""
    if next := request.GET.get("next"):
        q = QueryDict(mutable=True)
        q["next"] = next
        query = f"?{q.urlencode()}"

    return redirect(f"{reverse('oidc_authentication_init')}{query}")
